@@ -158,16 +158,16 @@ class SDK:
             print(e)
             return False
 
-    def send_command(self, instrument_id, command_invocation, response_format=None, convert_result_to=None):
+    def send_command(self, instrument_id, command_invocation, command_result_output_file=None, response_format=None, convert_result_to=None):
         if response_format == SDK_RESPONSE_FORMAT_JSON or (
                 response_format == None and self._default_response_format == SDK_RESPONSE_FORMAT_JSON):
             # If response format is json convert_result_to is ignored
-            return self._client_protocol.send_command_and_result_as_json_string(instrument_id, command_invocation)
+            return self._client_protocol.send_command_and_result_as_json_string(instrument_id, command_invocation, command_result_output_file)
 
         command_execution_result = self._client_protocol.send_command(
-            instrument_id, command_invocation)
+            instrument_id, command_invocation, command_result_output_file)
 
-        if not convert_result_to:
+        if not convert_result_to or command_result_output_file:
             return command_execution_result
 
         original_value = command_execution_result["value"]
@@ -205,6 +205,16 @@ class SDK:
         with open(file_target_name, "wb") as file:
             file.write(file_bytes)
 
+    def get_directory_structure(self, remote_path, response_format=None):
+        directory_structure = self._client_protocol.get_directory_structure_as_json_string(remote_path)
+        return self.__format_response(directory_structure, response_format)
+
+    def create_directory(self, remote_path, new_directory):
+        return self._client_protocol.create_directory(self, remote_path, new_directory)
+
+    def delete_directory(self, remote_path):
+        return self._client_protocol.delete_directory(remote_path)
+
     def execute_bash_command(self, command, capture_stdout=False, capture_stderr=False):
         return self._client_protocol.execute_bash_command(command, capture_stdout, capture_stderr)
 
