@@ -21,6 +21,9 @@ COMMAND_GET_INSTRUMENT_COMMANDS = "GET_INSTRUMENT_COMMANDS"
 COMMAND_VALIDATE_COMMAND = "VALIDATE_COMMAND"
 COMMAND_SEND_COMMAND = "SEND_COMMAND"
 COMMAND_GET_FILE = "GET_FILE"
+COMMAND_GET_DIRECTORY_STRUCTURE = "GET_DIRECTORY_STRUCTURE"
+COMMAND_CREATE_DIRECTORY = "CREATE_DIRECTORY"
+COMMAND_DELETE_DIRECTORY = "DELETE_DIRECTORY"
 COMMAND_SEND_FILE = "SEND_FILE"
 COMMAND_DELETE_FILE = "DELETE_FILE"
 COMMAND_EXECUTE_BASH = "EXECUTE_BASH"
@@ -167,27 +170,43 @@ class ClientProtocol:
         log.debug("[LATENCY_MEASURE][FINISH][{}][ELAPSED={} seconds]".format(
             'validate_command', te-ts))
 
-    def send_command(self, id, command):
+    def send_command(self, id, command, command_result_output_file):
         id = str(id)
-        json_str = self.send_command_and_result_as_json_string(id, command)
+        json_str = self.send_command_and_result_as_json_string(
+            id, command, command_result_output_file)
+
+        if not json_str and command_result_output_file:
+            return None
+
         command_execution_result_dict = json.loads(json_str)
+        command_execution_value = command_execution_result_dict["value"]
+        command_execution_type = command_execution_result_dict["type"]
+
+        if not command_execution_value:  # output value was saved in server
+            return command_execution_result_dict
+
         # BYTES are sent as a base64 string
-        if command_execution_result_dict["type"] == "BYTES":
-            command_execution_result_dict["value"] = base64.b64decode(
-                command_execution_result_dict["value"])
+        if command_execution_type == "BYTES":
+            command_execution_value = base64.b64decode(command_execution_value)
         return command_execution_result_dict
 
-    def send_command_and_result_as_json_string(self, id, command):
+    def send_command_and_result_as_json_string(self, id, command, command_result_output_file):
         id = str(id)
         log.debug("[LATENCY_MEASURE][INIT][{}][command={}]".format(
             'send_command', command))
         ts = time()
         self._message_protocol.send_msg(COMMAND_SEND_COMMAND)
-        self._message_protocol.send_msg(id)
-        self._message_protocol.send_msg(command)
+        command_execution_request = {
+            "instrument_id": id,
+            "command_invocation": command,
+            "command_result_output_file": command_result_output_file
+        }
+        self._message_protocol.send_msg(json.dumps(command_execution_request))
         response_type = self._message_protocol.receive_msg()
         if self.__is_valid_response(response_type):
-            command_execution_result_json_str = self._message_protocol.receive_msg()
+            command_execution_result_json_str = None
+            if not command_result_output_file: # if result was saved in Server this message is not sent
+                command_execution_result_json_str = self._message_protocol.receive_msg()
             te = time()
             log.debug("[LATENCY_MEASURE][FINISH][{}][command={}][ELAPSED={} seconds]".format(
                 'send_command', command, te-ts))
@@ -217,7 +236,8 @@ class ClientProtocol:
             raise InvalidPathException(err)
 
         te = time()
-        log.debug("[LATENCY_MEASURE][FINISH][{}][ELAPSED={} seconds]".format('send_file', te - ts))
+        log.debug("[LATENCY_MEASURE][FINISH][{}][ELAPSED={} seconds]".format(
+            'send_file', te - ts))
 
         return response
 
@@ -249,16 +269,81 @@ class ClientProtocol:
 
         if not self.__is_valid_response(response_type):
             error_message = self._message_protocol.receive_msg()
-            log.error("Error requesting remote file '{}' : {}".format(remote_file_name, error_message))
+            log.error("Error requesting remote file '{}' : {}".format(
+                remote_file_name, error_message))
             raise OpenLISAException(error_message)
 
         file_bytes = self._message_protocol.receive_msg(decode=False)
 
         te = time()
-        log.debug("[LATENCY_MEASURE][FINISH][{}][ELAPSED={} seconds]".format('get_file', te - ts))
+        log.debug("[LATENCY_MEASURE][FINISH][{}][ELAPSED={} seconds]".format(
+            'get_file', te - ts))
 
         return file_bytes
 
+    def delete_file(self, remote_file):
+        log.debug("[LATENCY_MEASURE][INIT][{}]".format('delete_file'))
+        ts = time()
+        self._message_protocol.send_msg(COMMAND_DELETE_FILE)
+        self._message_protocol.send_msg(remote_file)
+        response_type = str(self._message_protocol.receive_msg())
+
+        if not self.__is_valid_response(response_type):
+            error_message = self._message_protocol.receive_msg()
+            log.error("Error deleting remote file '{}' : {}".format(
+                remote_file, error_message))
+            raise OpenLISAException(error_message)
+
+        te = time()
+        log.debug("[LATENCY_MEASURE][FINISH][{}][ELAPSED={} seconds]".format(
+            'delete_file', te - ts))
+
+        return
+
+    def get_directory_structure_as_json_string(self, remote_path):
+        log.debug("[LATENCY_MEASURE][INIT][{}]".format('get_directory_structure'))
+        ts = time()
+        self._message_protocol.send_msg(COMMAND_GET_DIRECTORY_STRUCTURE)
+        self._message_protocol.send_msg(remote_path)
+        response_type = str(self._message_protocol.receive_msg())
+
+        if not self.__is_valid_response(response_type):
+            error_message = self._message_protocol.receive_msg()
+            log.error("Error getting remote directory structure '{}' : {}".format(
+                remote_path, error_message))
+            raise OpenLISAException(error_message)
+        else:
+            directory_structure_as_json_string = self._message_protocol.receive_msg()
+            te = time()
+            log.debug("[LATENCY_MEASURE][FINISH][{}][ELAPSED={} seconds]".format(
+                'get_directory_structure', te - ts))
+            return directory_structure_as_json_string
+
+
+    def get_directory_structure(self, remote_path):
+        return json.loads(self.get_directory_structure_as_json_string(remote_path))
+
+    def create_directory(self, remote_path, new_directory_name):
+        log.debug("[LATENCY_MEASURE][INIT][{}]".format('create_directory'))
+        ts = time()
+        self._message_protocol.send_msg(COMMAND_CREATE_DIRECTORY)
+        self._message_protocol.send_msg(remote_path)
+        self._message_protocol.send_msg(new_directory_name)
+        response_type = str(self._message_protocol.receive_msg())
+
+        if not self.__is_valid_response(response_type):
+            error_message = self._message_protocol.receive_msg()
+            log.error("Error creating remote directory '{}' : {}".format(
+                remote_path, error_message))
+            raise OpenLISAException(error_message)
+
+        te = time()
+        log.debug("[LATENCY_MEASURE][FINISH][{}][ELAPSED={} seconds]".format(
+            'create_directory', te - ts))
+
+        return
+
+
     def execute_bash_command(self, command, capture_stdout, capture_stderr):
         log.debug("[LATENCY_MEASURE][INIT][{}]".format('execute_bash_command'))
         ts = time()
@@ -269,7 +354,8 @@ class ClientProtocol:
         self._message_protocol.send_msg(str(capture_stdout))
         self._message_protocol.send_msg(str(capture_stderr))
         status_code = str(self._message_protocol.receive_msg())
-        log.info("Status code after remote bash command execution: {}".format(status_code))
+        log.info(
+            "Status code after remote bash command execution: {}".format(status_code))
 
         if capture_stdout:
             stdout = str(self._message_protocol.receive_msg())
@@ -280,7 +366,8 @@ class ClientProtocol:
             log.debug("Remote execution command stderr: {}".format(stderr))
 
         te = time()
-        log.debug("[LATENCY_MEASURE][FINISH][{}][ELAPSED={} seconds]".format('execute_bash_command', te - ts))
+        log.debug("[LATENCY_MEASURE][FINISH][{}][ELAPSED={} seconds]".format(
+            'execute_bash_command', te - ts))
 
         return status_code, stdout, stderr
 
